#include "sgio_socket.h"
#include "command.h"
#include "agent_commands.h"
#include "tcl_commands.h"
#include "null.h"
#include "sio_soar_proxy.h"
#include "sio_agent_proxy.h"
#include "tuple_converter.h"
#include "sgio_messages.h"
#include "tsi.h"

#include <tcl.h>
#include <tk.h>
#include <cstdio>
#include <iostream>
#include <strstream>

using namespace std;

TSI g_tsi("soarside.log");
SIOSoarProxy* g_soarProxy = null;
void TclLoop();

const char* usage = "soarside.exe <port>";

int 
main(int argc,char* argv[]) {
     int port = 6969;
     if(argc == 1) {
          // Everything is ok
     }
     else if(argc == 2) {
          std::istrstream argvStream(argv[1]);
          argvStream >> port;
          if(!argvStream) {
               std::cerr << "Could not read port number!\n";
               std::cerr << usage << "\n";
               return -1;
          }
          

     }
     else {
          std::cerr << "Too many parameters!\n";
          std::cerr << usage << "\n";
          return -1;
     }


     sgio::Socket* client = new sgio::Socket();
     try {
          cout << "Awaiting Connection on port " << port << endl;
          sgio::ServerSocket serverSocket(port);

          serverSocket.Accept(client);
     }
     catch(sgio::SocketException& se) {
          cerr << se.what() << "\n";
          return -1;
     }

     cout << "Connect accepted" << endl;
     g_soarProxy = new SIOSoarProxy(std::auto_ptr<sgio::Socket>(client));
     TclLoop();
	 delete g_soarProxy;
     return 0;
}

int 
ProcessInputLinkCmd(ClientData clientData,Tcl_Interp* interp,int argc,char* argv[]) throw() {
	if(argc != 2) {
		interp->result = "Usage: processInputLink <agentName>";
		return TCL_ERROR;
	}

     g_soarProxy->RunAgentCommand(argv[1],DoWMEChangesCommand(interp));
	return TCL_OK;
}

int 
ProcessOutputLinkCmd(ClientData clientData,Tcl_Interp* interp,int argc,char* argv[]) throw() {
	if(argc != 3) {
		interp->result = "Usage: processOutputLink <agentName> <listName>";
		return TCL_ERROR;
	}
	int numElements;
     char** elements;
     int result = Tcl_SplitList(interp,argv[2],&numElements,&elements);
     if(result == TCL_OK) {
          TupleConverter tc;
          for(int i = 0; i < numElements; ++i) {
               int check;
               char** tuple;
               result = Tcl_SplitList(interp,elements[i],&check,&tuple);
               if(result == TCL_OK && check == 3) {
                    tc.AddTuple(tuple[0],tuple[1],tuple[2]);
			     Tcl_Free(reinterpret_cast<char*>(tuple));
               }
               else if(result == TCL_OK) {
                    Tcl_Free(reinterpret_cast<char*>(tuple));
               }
          }
          Tcl_Free(reinterpret_cast<char*>(elements));
          g_soarProxy->RunAgentCommand(argv[1],
               ReportCommands(tc.GetCommands()));
          return TCL_OK;
      }
      else {
         return TCL_ERROR;
     }
}

int 
ManageInputLinkCmd(ClientData clientData,Tcl_Interp* interp,int argc,char* argv[]) throw() {
   if(argc == 3) {
	   char cmdbuf[256];
	   sprintf(cmdbuf,"processInputLink %s\n",argv[1]);
	   Tcl_Eval(interp,cmdbuf);
      return TCL_OK;
   }
   else {
      interp->result = "Wrong Number of arguments";
      return TCL_ERROR;
   }
}

int 
ManageOutputLinkCmd(ClientData clientData,Tcl_Interp* interp,int argc,char* argv[]) throw() {
   if(argc == 4) {
      std::ostrstream out;
      out << "processOutputLink " << argv[1] << " { " <<
              argv[3] << " }\n " << std::ends;
      Tcl_Eval(interp,out.str());
	  out.freeze(0);
      return TCL_OK;
   }
   else {
      interp->result = "Wrong Number of arguments";
      return TCL_ERROR;
   }
}



void 
TclLoop() {
    Tcl_Interp* interp = Tcl_CreateInterp();
    if(Tcl_Init(interp) == TCL_ERROR) {
	    Tcl_DeleteInterp(interp);
        return;
    }
    if(Tk_Init(interp) == TCL_ERROR) {
		Tcl_DeleteInterp(interp);
        return;
    }
    char setargc[] = "set argc 0";
    char setargv[] = "set argv {}";
    Tcl_Eval(interp,setargc);
    Tcl_Eval(interp,setargv);
    Tcl_CreateCommand(interp,"manageInputLink",ManageInputLinkCmd,null,null);
    Tcl_CreateCommand(interp,"manageOutputLink",ManageOutputLinkCmd,null,null);
	Tcl_CreateCommand(interp,"processInputLink",ProcessInputLinkCmd,null,null);
	Tcl_CreateCommand(interp,"processOutputLink",ProcessOutputLinkCmd,null,null);
	if(Tcl_Eval(interp,"cd soar") == TCL_ERROR) {
		std::cout << interp->result << std::endl;
        Tcl_DeleteInterp(interp);
		return;
	}
	if(Tcl_Eval(interp,"source start-soar.tcl") == TCL_ERROR) {
		std::cout << interp->result << std::endl;
        Tcl_DeleteInterp(interp);
		return;
	}
    while(!g_soarProxy->GetQuit()) {
        while(g_soarProxy->IsATclCommandPending()) {
            std::auto_ptr<TclCommand> tclCommand = g_soarProxy->GetTclCommand();
            tclCommand->Execute(interp);
        }
        Tcl_DoOneEvent(TCL_DONT_WAIT);
    }
}
