#pragma warning(disable:4786)

#include "receiver.h"
#include "thread.h"
#include "sgio_socket.h"
#include "sgio_messages.h"
#include "tcl_commands.h"
#include "sio_soar_proxy.h"
#include "sio_agent_proxy.h"
#include "null.h"
#include "sender.h"
#include "agent_commands.h"

#include <algorithm>
#include <string>
#include <iostream>
#include <process.h>
#include <windows.h>

using namespace sgio;

extern SIOSoarProxy* g_soarProxy;

Receiver::Receiver(SIOSoarProxy* inSoarProxy,sgio::Socket* inSocket,Sender* inSender) 
: d_soarProxy(inSoarProxy),d_sock(inSocket),d_quit(false),
  d_sender(inSender) {
     Thread::BeginThread(thread_func,this);
}

Receiver::~Receiver() {
     SetQuit();
     d_quitSignal.Wait();
}

void Receiver::ThreadFunc() {
     try {
	     MessageLoop();
     }
     catch(std::exception& e) {
          std::cerr << e.what() << std::endl;
     }
	 catch(...) {
		 std::cerr << "An unknown exception was thrown!" << std::endl;
	 }
     d_quitSignal.Set();
}

void Receiver::MessageLoop() {
	while(!GetQuit()) {
		int message_type;
		(*d_sock) >> message_type;
		switch(message_type) {
			case e_mt_add_named_agent: 
				HandleAddAgent();
				break;
			case e_mt_add_data_tuple:
				HandleAddDataTuple();
				break;
			case e_mt_add_tree_tuple:
				HandleAddTreeTuple();
				break;
			case e_mt_add_plain_tuple:
				HandleAddPlainTuple();
				break;
			case e_mt_commit_wme_changes:
				HandleCommitWmeChanges();
				break;
			case e_mt_load_productions:
				HandleLoadProductions();
				break;
			case e_mt_load_retenet:
				HandleLoadReteNet();
				break;
			case e_mt_remove_agent:
				HandleRemoveAgent();
				break;
			case e_mt_remove_tuple:
				HandleRemoveTuple();
				break;
			case e_mt_run_til_output:
				HandleRunTilOutput();
				break;
			case e_mt_save_retenet:
				HandleSaveReteNet();
				break;
			case e_mt_set_watch_level:
				HandleSetWatchLevel();
				break;
            case e_mt_run_agent_til_output:
                HandleRunAgentTilOutput();
                break;
			case e_mt_disconnect_from_soar:
				HandleDisconnect();
				return;
			case e_mt_step:
				HandleStep();
				break;
            case e_mt_step_agent:
                HandleStepAgent();
                break;
			default:
				//cerr << "That message type is not handled, update SIOServer MessageLog";
			break;
		}
	}
}

void 
Receiver::HandleAddAgent() {
	AddNamedAgentMessage addNamedAgentMsg;
	addNamedAgentMsg.Receive(d_sock);

    d_soarProxy->CreateAgent(addNamedAgentMsg.GetAgentName().c_str());
}

void 
Receiver::HandleAddDataTuple() {
	AddDataTupleMessage adtm;
	adtm.Receive(d_sock);

     d_soarProxy->RunAgentCommand(adtm.GetAgentName().c_str(),
          AddDataWMECommand(adtm.GetParentId(), adtm.GetAttribute().c_str(),
                            adtm.GetValue().c_str(),adtm.GetWmeId()));
}

void 
Receiver::HandleAddPlainTuple() {
   AddPlainTupleMessage aptm;
   aptm.Receive(d_sock);

   d_soarProxy->RunAgentCommand(aptm.GetAgentName().c_str(),
        AddPlainWMECommand(aptm.GetParent().c_str(),aptm.GetAttribute().c_str(),aptm.GetValue().c_str()));
}


void 
Receiver::HandleAddTreeTuple() {
	AddTreeTupleMessage attm;
	attm.Receive(d_sock);

     d_soarProxy->RunAgentCommand(attm.GetAgentName().c_str(),
          AddIdWMECommand(attm.GetParentId(),attm.GetAttribute().c_str(),
                          attm.GetNewId(),attm.GetWmeId()));
}

void 
Receiver::HandleLoadProductions() {
	LoadProductionsMessage loadProductionsMsg;
	loadProductionsMsg.Receive(d_sock);

	std::auto_ptr<TclCommand> loadProductionsCmd( 
				new LoadProductionsTclCmd(loadProductionsMsg.GetAgentName(),
				                          loadProductionsMsg.GetPath()));
	d_soarProxy->PostTclCommand(loadProductionsCmd);
}

void 
Receiver::HandleLoadReteNet() {
	LoadReteNetMessage loadReteNetMsg;
	loadReteNetMsg.Receive(d_sock);
	std::auto_ptr<TclCommand> loadReteNetCmd(
				new LoadReteNetTclCmd(loadReteNetMsg.GetAgentName(),
							          loadReteNetMsg.GetPath()));
	d_soarProxy->PostTclCommand(loadReteNetCmd);
}

void 
Receiver::HandleRemoveAgent() {
	RemoveAgentMessage removeAgentMsg;
	removeAgentMsg.Receive(d_sock);
	
    std::auto_ptr<TclCommand> removeAgentCmd(
                                new RemoveAgentTclCmd(removeAgentMsg.GetAgentName())); 
	d_soarProxy->PostTclCommand(removeAgentCmd);
}

void 
Receiver::HandleRemoveTuple() {
	RemoveTupleMessage rtm;
	rtm.Receive(d_sock);

     d_soarProxy->RunAgentCommand(rtm.GetAgentName().c_str(),
                                  RemoveWMECommand(rtm.GetWmeId()));
}

void 
Receiver::HandleCommitWmeChanges() {
   CommitWmeChangesMessage cwcm;
   cwcm.Receive(d_sock);

   d_soarProxy->RunAgentCommand(cwcm.GetAgentName().c_str(),CommitCommand());
}

void 
Receiver::HandleRunTilOutput() {
	RunTilOutputMessage runMsg;
	runMsg.Receive(d_sock);

     std::string agentName = d_soarProxy->GetAnyAgentName();
     if(!agentName.empty()) {
         std::auto_ptr<TclCommand> runCmd(
                            new RunTilOutputTclCmd(agentName,runMsg.GetMaxDecisions()));
	     d_soarProxy->PostTclCommand(runCmd);
     }
	
}


void 
Receiver::HandleStep() {
	StepMessage stepMsg;
	stepMsg.Receive(d_sock);

     std::string agentName = d_soarProxy->GetAnyAgentName();
     if(!agentName.empty()) {
         std::auto_ptr<TclCommand> stepCmd(
                           new StepTclCmd(agentName));
	     d_soarProxy->PostTclCommand(stepCmd);
     }	
}


void 
Receiver::HandleSaveReteNet() {
	SaveReteNetMessage saveReteNetMsg;
	saveReteNetMsg.Receive(d_sock);
		

	std::auto_ptr<TclCommand> saveReteNetCmd(
                            new SaveReteNetTclCmd(saveReteNetMsg.GetAgentName(),
							                       saveReteNetMsg.GetPath()));
	d_soarProxy->PostTclCommand(saveReteNetCmd);
}

void
Receiver::HandleSetWatchLevel() {
	SetWatchLevelMessage setWatchLevelMsg;
	setWatchLevelMsg.Receive(d_sock);

	std::auto_ptr<TclCommand> setWatchLevelCmd(
				new SetWatchLevelTclCmd(setWatchLevelMsg.GetAgentName(),
				                     setWatchLevelMsg.GetWatchLevel()));
	d_soarProxy->PostTclCommand(setWatchLevelCmd);
}

void
Receiver::HandleRunAgentTilOutput() {
    RunAgentTilOutputMessage runAgentTilOutputMsg;
    runAgentTilOutputMsg.Receive(d_sock);

    std::auto_ptr<TclCommand> runAgentTilOutputCmd(
                new RunAgentTilOutputTclCmd(runAgentTilOutputMsg.GetAgentName()));
    d_soarProxy->PostTclCommand(runAgentTilOutputCmd);
}


void
Receiver::HandleStepAgent() {
    StepAgentMessage stepAgentMsg;
    stepAgentMsg.Receive(d_sock);

    std::auto_ptr<TclCommand> stepAgentCmd(
                new StepAgentTclCmd(stepAgentMsg.GetAgentName()));
    d_soarProxy->PostTclCommand(stepAgentCmd);

}


void 
Receiver::HandleDisconnect() {
	DisconnectFromSoarMessage disconnect;
	disconnect.Receive(d_sock);
    d_sender->PostNetMessage(new ConfirmDisconnectMessage());
    g_soarProxy->Quit();
}

void 
Receiver::thread_func(void* self) {
	Receiver* receiver = static_cast<Receiver*>(self);
    SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_TIME_CRITICAL);
	receiver->ThreadFunc();
}
