#ifndef INCLUDED_RECEIVER_H
#define INCLUDED_RECEIVER_H

#include <list>
 #include "thread.h"

namespace sgio {
	class Socket;
}

class Sender;
class SIOSoarProxy;

/**
  A Receiver is a class that sits and waits on the socket so
  you don't have to, after it grabs and interprets a Message
  it will either make the corresponding calls into the global
  tcl_queue or manipulate information within SIOProxyAgent
  so that the Soar callbacks will get the new info.

  Every sender object also has a receiver that sends information
  over the socket

  Since this socket is responsible for more than one agent, it
  also keeps track of agents that it is manipulating

  This object must be dynamically allocated

  @author Brad Jones
*/
class Receiver {
public:
     /**
          This creates a Receiver object and sets the reference
          count to one

          @param inSocket the socket to listen on, this object
                 owns this socket, and is responsible for delete'ing
                 it
     */
	Receiver(SIOSoarProxy* inSoarProxy,sgio::Socket* inSocket,Sender* inSender);


     ~Receiver();


private:

	// NOT IMPLEMENTED
	Receiver();
	Receiver(const Receiver&);
	Receiver& operator=(const Receiver&);

     // Helper Functions so that I don't have to write the locking code
     void SetQuit() { 
          // Sets quit to true
          Thread::Monitor m(&d_quitLock);
          d_quit = true;
     }

     
     bool GetQuit() {
          Thread::Monitor m(&d_quitLock);
          return d_quit;
     }
	
	// Thread Functions
	static void thread_func(void* self);
	
	// Functions called by the thread
    void ThreadFunc();
    void MessageLoop();
    void HandleAddAgent();
    void HandleAddDataTuple();
    void HandleAddTreeTuple();
    void HandleAddPlainTuple();
    void HandleCommitWmeChanges();
    void HandleLoadProductions();
    void HandleLoadReteNet();
    void HandleRemoveAgent();
    void HandleRemoveTuple();
    void HandleRunTilOutput();
    void HandleSaveReteNet();
	void HandleSetWatchLevel();
    void HandleRunAgentTilOutput(); 
    void HandleDisconnect();
	void HandleStep();
	void HandleStepAgent(); 

     // The socket which this function operaters
	sgio::Socket* d_sock;

     Sender*             d_sender;
     Thread::SingleEvent d_quitSignal;
     Thread::Mutex       d_quitLock;
     bool                d_quit;


     // The proxy agent which should be notified of certain commands
     SIOSoarProxy* d_soarProxy;
};


#endif // INCLUDED_RECEIVER_H
