#pragma warning(disable:4786)

#include "sio_agent_proxy.h"
#include "command.h"
#include "sgio_messages.h"
#include <iostream>
#include <tcl.h>
#include <sstream>
#include "null.h"
#include "sender.h"
#include "tsi.h"
#include "sgio_frame.h"


using namespace std;

extern TSI g_tsi;

SIOAgentProxy::SIOAgentProxy(const char* inName,Sender* inSender) 
: name(inName), sender(inSender) {
	idToSoarId[1] = "I2";
}

SIOAgentProxy::~SIOAgentProxy() {
}


void 
SIOAgentProxy::AddWME(int simsideId,const std::string& attribute,const std::string& value,int simsideTimeTag) {
    std::auto_ptr<sgio::WMChange> adw(new sgio::AddDataWME(simsideId,attribute,value,simsideTimeTag));
    building.Add(adw);
}

void 
SIOAgentProxy::AddWME(int simsideId,const std::string& attribute,int childSimsideId,int simsideTimeTag) {
    std::auto_ptr<sgio::WMChange> atw(new sgio::AddTreeWME(simsideId,attribute,childSimsideId,simsideTimeTag));
    building.Add(atw);
}

void 
SIOAgentProxy::AddWME(const std::string& soarId,const std::string& attribute,const std::string& value) {
    std::auto_ptr<sgio::WMChange> apw(new sgio::AddPlainWME(soarId,attribute,value));
    building.Add(apw);
}

void 
SIOAgentProxy::Commit() {
     Thread::Monitor m(&completeLock);
     complete.Merge(&building);
}

void 
SIOAgentProxy::RemoveWME(int inWmeId) {
    std::auto_ptr<sgio::WMChange> rw(new sgio::RemoveWME(inWmeId));
    building.Add(rw);
}

void 
SIOAgentProxy::DoWMChanges() {
	Thread::Monitor m(&completeLock);
    complete.Execute(this);
}


void 
SIOAgentProxy::DoAddWME(int simsideId,const std::string& attribute,int childSimsideId,int simsideTimeTag) {
	misIter i = idToSoarId.find(simsideId);
	if(i == idToSoarId.end()) {
          cout << "Couldn't find id for wme: (" << simsideId << "," << attribute << "," << childSimsideId << "): " << simsideTimeTag << "\n";
		return;
	}

     string stringId;
     int timetag = g_tsi.AddWme(interp,name.c_str(),i->second.c_str(),attribute.c_str(),&stringId);
     if(timetag != -1) {
          wmeIdToTimeTag[simsideTimeTag] = timetag;
		  idToSoarId[childSimsideId] =  stringId;
          wmeIdToId[simsideTimeTag] = childSimsideId;     
     }
     else {
		cout << "add-wme call failed: " << interp->result << endl;
	}
}

void 
SIOAgentProxy::DoAddWME(int simsideId,const std::string& attribute,const std::string& value,int simsideTimeTag) {
	misIter i = idToSoarId.find(simsideId);
	if(i == idToSoarId.end()) {
		cout << "Couldn't find id for wme: (" << simsideId << "," << attribute << "," << value << "): " << simsideTimeTag << "\n";
		return;
	}

     int timetag = g_tsi.AddWme(interp,name.c_str(),i->second.c_str(),attribute.c_str(),value.c_str());
     if(timetag != -1) {
		wmeIdToTimeTag[simsideTimeTag] = timetag;
     }
	else {
		cout << "add-wme call failed: " << interp->result << endl;
	}
}

void 
SIOAgentProxy::DoAddWME(const std::string& soarId,const std::string& attribute,const std::string& value) {
     if(g_tsi.AddWme(interp,name.c_str(),soarId.c_str(),attribute.c_str(),value.c_str()) == -1) {
      cout << "add-wme call failed: " << interp->result << endl;
   }
}
	
void 
SIOAgentProxy::DoRemoveWME(int simsideTimeTag) {
	miiIter i = wmeIdToTimeTag.find(simsideTimeTag);
	if(i == wmeIdToTimeTag.end()) {
		cout << "Couldn't find timetag " << simsideTimeTag << endl;
		return;
	}

	int timetag = i->second;
     if(g_tsi.RemoveWme(interp,name.c_str(),timetag)) {
	     wmeIdToTimeTag.erase(i);
          miiIter j = wmeIdToId.find(simsideTimeTag);
          if(j != wmeIdToId.end()) {
               misIter k = idToSoarId.find(j->second);
               if(k != idToSoarId.end()) {
                    idToSoarId.erase(k);
               }
               wmeIdToId.erase(j);
          }
	}
	else {
		cout << "Error Removing TimeTag: " << interp->result << endl;
	}
}



void 
SIOAgentProxy::AddCommand(const Command& inCommand) {
    /*
        Did we see this command last time
    */
    std::set<std::string>::iterator i = commandsSeenLastTime.find(inCommand.GetSoarId());
    if(i == commandsSeenLastTime.end()) {
        // No, so create a new command message
        std::auto_ptr<sgio::CommandMessage> cmdMsg(new sgio::CommandMessage(name.c_str(),
                        inCommand.GetCommandName().c_str(),inCommand.GetSoarId().c_str()));

        // Fill in the parameters also
        const Command::ParamList& paramList = inCommand.GetParameterList();
        Command::ParamList::const_iterator end = paramList.end();
        for(Command::ParamList::const_iterator ci = paramList.begin(); ci != end; ++ci) {
            cmdMsg->AddParameter(ci->first,ci->second);
        }

        // Send it
        sender->PostNetMessage(cmdMsg.release());
		//inCommand.Describe(cout);
     }
     commandsSeenThisTime.insert(inCommand.GetSoarId());
}


void 
SIOAgentProxy::EndCommands() {
    commandsSeenLastTime.clear();
    commandsSeenLastTime.swap(commandsSeenThisTime);
}