'\"
'\" Soar man page 
'\"
.so man.macros
.HS sp soar
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
sp \- Define a Soar production
.SH SYNOPSIS
\fBsp \fIrule\fR
.BE

.SH DESCRIPTION
.PP
This command defines a new Soar production.  \fIrule\fR 
is a single argument parsed by the Soar kernel, so it should
be enclosed in curly braces to avoid being parsed by Tcl.  
The
overall syntax of a \fIrule\fR is as follows:
.DS
   \fIname\fR 
       [\fB"\fR\fIdocumentation-string\fR\fB"\fR] 
       [\fIFLAG\fR*]
       \fILHS\fR
       \fB-->\fR
       \fIRHS\fR
.DE
.PP
The first element of a \fIrule\fR is its \fIname\fR.  Conventions
for \fInames\fR are given in the Soar Users Manual.  If given,
the \fIdocumentation-string\fR must be enclosed in double quotes.
Optional flags define the type of rule and the form of support its
right-hand side assertions will receive.  The specific flags are
listed in a separate section below.  The \fILHS\fR defines the
left-hand side of the production and specifies the conditions 
under which the rule can be fired.  Its syntax is given in detail
in a subsequent section.  The \fB-->\fR symbol serves to separate
the \fILHS\fR and \fIRHS\fR portions.
The \fIRHS\fR defines the right-hand side of
the production and specifies the assertions to
be made and the actions to be performed when the rule fires.  The
syntax of the allowable right-hand side actions are given in a later
section.  The Soar Users Manual gives an elaborate discussion of
the design and coding of productions.  Please see that reference
for tutorial information about productions.
.PP
More complex productions can be formed by surrounding the \fIrule\fR with 
double quotes instead of curly braces.  This enables variable and 
command result substitutions in productions.  
If another production with the same name already exists, it is 
excised, and the new production is loaded.

.SH RULE FLAGS
.PP
The optional \fIFLAG\fRs are given below.  Note that these switches
are preceeded by a colon instead of a dash -- this is a Soar parser
convention.
.TP
\fB:o-support\fR
specifies that all the RHS actions are to be given 
o-support when the production fires
.TP
\fB:no-support\fR
specifies that all the RHS actions are only 
to be given i-support when the production fires
.TP
\fB:default\fR
specifies that this production is a default 
production (this matters for \fBexcise -task\fR and \fBwatch task\fR)
.TP
\fB:chunk\fR
specifies that this production is a chunk (this 
matters for \fBlearn trace\fR)

.SH LHS GRAMMAR
Grammar for left hand sides of productions:
.DS
\fR   \fIlhs\fR ::= \fIcond\fR+
   \fIcond\fR ::= \fIpositive_cond\fR  | - \fIpositive_cond\fR
   \fIpositive_cond\fR ::= \fIconds_for_one_id\fR  | { \fIcond\fR+ }
   \fIconds_for_one_id\fR ::= ( [state |impasse] [\fIid_test\fR] \fIattr_value_tests\fR* )
   \fIid_test\fR ::= \fItest\fR
   \fIattr_value_tests\fR ::= [-] ^ \fIattr_test\fR [.\fIattr_test\fR]* \fIvalue_test\fR*
   \fIattr_test\fR ::= \fItest\fR
   \fIvalue_test\fR ::= \fItest\fR [+]  | \fIconds_for_one_id\fR [+]

   \fItest\fR ::= \fIconjunctive_test\fR  | \fIsimple_test\fR
   \fIconjunctive_test\fR ::= { \fIsimple_test\fR+ }
   \fIsimple_test\fR ::= \fIdisjunction_test\fR  | \fIrelational_test\fR
   \fIdisjunction_test\fR ::= << \fIconstant\fR* >>
   \fIrelational_test\fR ::= [\fIrelation\fR] \fIsingle_test\fR
   \fIrelation\fR ::= <>  | <  | >  | <=  | >=  | =  | <=>
   \fIsingle_test\fR ::= variable  | \fIconstant\fR
   \fIconstant\fR ::= sym_constant  | int_constant  | float_constant
.DE
.SH RHS GRAMMAR
Grammar for right hand sides of productions:
.DS
\fR   \fIrhs\fR ::= \fIrhs_action\fR*
   \fIrhs_action\fR ::= ( variable \fIattr_value_make\fR+ )  | \fIfunction_call\fR
   \fIfunction_call\fR ::= ( \fIfunction_name\fR \fIrhs_value\fR* )
   \fIfunction_name\fR ::= sym_constant  | +  | -
   \fIrhs_value\fR ::= \fIconstant\fR  | \fIfunction_call\fR  | variable
   \fIconstant\fR ::= sym_constant  | int_constant  | float_constant
   \fIattr_value_make\fR ::= ^ \fIrhs_value\fR [.\fIrhs_value\fR]* \fIvalue_make\fR+
   \fIvalue_make\fR ::= \fIrhs_value\fR \fIpreferences\fR

   \fIpreferences\fR ::= [,] | \fIpreference-specifier\fR+
   \fIpreference-specifier\fR ::= \fInaturally-unary-preference\fR [,]
                             | \fIforced-unary-preference\fR
                             | \fIbinary-preference\fR \fIrhs_value\fR [,]
   \fInaturally-unary-preference\fR ::= +  | -  | !  | ~  | @
   \fIbinary-preference\fR ::= >  | =  | <  | &
   \fIforced-unary-preference\fR ::= \fIbinary-preference\fR {,  | )  | ^}
.DE

.SH EXAMPLES
There are many examples in the Soar Users Manual and the demos
subdirectory.  Here is a simple production to create a problem space.
It comes from the critter-world demo (see the file critter.tcl):
.DS
\fB
sp {critter*create*space*critter
   "Formulate the initial problem space"
  (state <s> ^superstate nil)
  -->
  (<s> ^name move-around ^problem-space <p>)
  (<p> ^name critter)}
\fR
.DE
.PP
The production above has the \fIname\fR 
\fBcritter*create*space*critter\fR.  It has a documentation string
that is surrounded by double quotes.  The \fILHS\fR is \fB(state <s> 
^superstate nil)\fR and indicates that this rule will match whenever
there is a state object that has the attribute-value pair
\fB^superstate 
nil\fR.  The \fB-->\fR arrow separates the left and right-hand sides.
The \fIRHS\fR consists of two lines.  The first asserts that the state
object is to be augmented with the name \fBmove-around\fR and a
problem space should be created.  The second line of the \fIRHS\fR 
indicates that this problem space should be named \fBcritter\fR.
.PP
New for Soar 8, is right-hand-side dot notation.  So this production
could also be written:
.DS
\fB
sp {critter*create*space*critter
   "Formulate the initial problem space"
  (state <s> ^superstate nil)
  -->
  (<s> ^name move-around ^problem-space.name critter)}
\fR
.DE
.PP
Here is a variant of the above example using 
double quotes instead of curly braces.  Double quotes are needed in order 
to imbed the value of the Tcl variable \fBsoar_agent_name\fR 
in the production.  The value of this variable is used to name
the problem-space created.
.DS
\fB
sp "critter*create*space*critter
   (state <s> ^superstate nil)
  -->
  (<s> ^name move-around ^problem-space <p>)
  (<p> ^name $soar_agent_name)"
\fR
.DE
.PP
The primary change in the rule is the last clause of the 
\fIRHS\fR.  In that clause, the Tcl variable \fBsoar_agent_name\fR
is expanded.  If this rule is given in an interpreter which has 
the variable \fBsoar_agent_name\fR set to \fBfred\fR, then the 
\fIRHS\fR would expand to the following before being sent to 
the Soar kernel to be parsed:
.DS
\fB
  (<p> ^name fred)
\fR
.DE
.PP
Please be aware that when using double quotes, both the dollar
sign (variable expansion) and square brackets (command result
substitution) will be interpreted by Tcl.  If these characters
(\fB$\fR, \fB[\fR, and \fB]\fR) are to be passed to the Soar 
production parser, they must be escaped (using a backslash) 
to avoid interpretation by Tcl.

.SH WARNINGS
.PP
The last production above does not contain a documentation string.  
If one were added, double quotes would need to be escaped to avoid 
premature termination of the rule:
.DS
\fB
sp "critter*create*space*critter
   \"Formulate the initial problem space\"
   (state ...
\fR
.DE

.SH SEE ALSO
\fBexcise\fR, \fBlearn\fR, \fBwatch\fR

