'\"
'\" Soar man page 
'\"
.so man.macros
.HS watch soar
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
watch \- Run-time tracing of Soar
.SH SYNOPSIS
\fBwatch\fR [\fIlevel\fR*] [-on | -off | -inc[lusive]]
.br
\fBwatch wmes\fR [-on | -off | -inc[lusive] | filter-options]
.br
\fBwatch\fR [ aliases | backtracing | loading ] [-on | -off ]
.br
\fBwatch\fR [ learning ] [-noprint | -print | -fullprint ]
.BE
.SH DESCRIPTION
.PP
The \fBwatch\fR command controls run-time tracing of Soar.
With no arguments, this command prints out the current watch 
status.  
The various \fIlevel\fRs are used to modify the current watch
settings.  Each \fIlevel\fR can be indicated with either a number or a
named flag as follows:
.DS
0  none
1  decisions
2  phases
3  productions
4  wmes
5  preferences
.DE
.PP
The numerical arguments (0 - 5) do not take any arguments or modifiers.
They inclusively turn on all levels up to the number specified.  To use
numerical arguments to turn off a level, specify a number which is
less than the level to be turned off.  For instance, to turn off watching
of productions, specify "watch 2" (or 1 or 0).  Numerical arguments are
provided for shorthand convenience.  For more detailed control over
the watch settings, the named arguments should be used.
.PP
For the named arguments, 
setting either the -\fBon\fR or -\fBoff\fR switch selectively turns on or
off only that setting.  Setting the -\fBinclusive\fR switch (which can be
abbreviated as -inc) or setting no flag at all has the effect of setting
all levels up to and including the level specified.  
For example,   \fBwatch productions -on\fR   selectively turns
on the 
tracing of production firings/retractions; \fBwatch productions -off\fR 
selectively turns it off again.  \fBwatch productions [-inc]\fR turns on the
tracing of productions and also turns on tracing of all levels below
productions: decisions and phases, too.  Individual watch parameters may 
be used to modify the inclusive settings as well, selectively turning on
or off any levels outside or inside the inclusive range.  


.SH OPTIONS
.PP
The following options are allowed (\fIstatus\fR is given as either
-\fBon\fR, -\fBoff\fR, or -\fBinclusive\fR, -\fBinc\fR is the default):
.TP
0 | \fBnone\fR  
turns off all printing about Soar's internals
.TP
1 | \fBdecisions\fR \fIstatus\fR 
controls whether state and operator decisions are printed as they are made
.TP
2 | \fBphases\fR \fIstatus\fR
controls whether decisions cycle phase names are printed as Soar executes.
.TP
3 | \fBproductions\fR \fIstatus\fR
\fRcontrols whether the names of
productions are printed as they fire and retract.  See the 
optional arguments below [-all|-chunks|-defaults|-justifications|-user] 
which specify which types of productions the \fIstatus\fR argument refers to.  
.TP
4 | \fBwmes\fR 
controls the printing of working memory elements that are added and deleted as
productions are fired and retracted.
.PP
.DS
To improve state tracing, the following filter-options
are available to \fBwatch wmes\fR for monitoring a specified
object-attribute-value triplet:
.DS
{-add-filter    \fItype filter\fR} |
{-remove-filter \fItype filter\fR} |
{-reset-filter  \fItype\fR       } |
{-list-filter   \fItype\fR       } 
.DE
.PP
where:
.DS
\fItype\fR   = -adds | -removes | -both
\fIfilter\fR = {\fIid\fR|*} {\fIattribute\fR|*} {\fIvalue\fR|*}
.DE
.DE
.PP
.TP
5 | \fBpreferences\fR \fIstatus\fR
controls whether the preferences 
generated by the traced productions are printed when 
those productions fire or retract.  When a production 
fires, all the preferences it generates are printed.  
When it retracts, only the ones being removed from
preference memory are printed (i.e., the i-supported ones).
.TP
\fB-nowmes | -timetags | -fullwmes\fR
controls the level of detail given about the wmes matched by productions 
whose firings and retractions are being traced.  Level 0|-nowmes means no
information about the wmes is printed.  Level 1|-timetags means the wme 
timetags are printed.  Level 2|-fullwmes means the whole wmes are printed.
.TP
\fB-all|-default|-user|-chunks|-justifications\fR \fI[modifier]\fR
allows user to selectively control printing by production *type*
as they fire and retract, instead of having to watch ALL productions
or none.  (NOTE:  these args can be abbreviated by the first
char ONLY, otherwise the full arg must be specified.)
The modifier can be one of:
.DS
\fB-noprint|-print|-fullprint\fR
.DE
.PP
.DS
-noprint prints nothing, 
-print prints the production name,
-fullprint prints the entire production. (not done as of 8.2)
.DE
.PP
.PP
The following options control printing of other Soar parameters and are
not part of the above set of levels which can be controlled as a group.
.TP
\fBaliases -on | -off\fR
controls the echoing of aliases as they are defined.
.TP
\fBbacktracing -on | -off\fR
controls the printing of backtracing information when a chunk or
justification is created.
.TP
\fBloading -on | -off \fR
controls the printing of '*' for each production loaded and a '#' for 
each production excised
.TP
\fBlearning -noprint | -print | -fullprint
controls the printing of
chunks/justifications as they are created. -noprint is
don't print anything, -print prints just the names,
-fullprint prints the entire chunk/justification.
.PP

.SH EXAMPLES
The most common uses of \fBwatch\fR are by using the numeric arguments
which indicate \fBwatch\fR levels:
.DS
\fBwatch 0\fR
.DE
which turns off all printing of Soar internals (same as watch none), or
.DS
\fBwatch 3\fR
.DE
which turns on printing of decisions, phases and productions (same as 
watch productions).  
Of course, individual options can be changed as well:
.DS
\fBwatch 1 wmes -on 
.DE
turns on printing of decisions and wmes, but not phases and productions.  
While
.DS
\fBwatch 4 phases -off
.DE
turns on printing of decisions, productions and wmes, and turns phases off.
.PP
To watch the firing and retraction of only USER productions:
.DS
\fBwatch productions -user -print\fR
.DE
.PP
whereas this command would watch decisions, phases and ALL productions:
.DS
\fBwatch productions\fR
.DE


.SH NOTE
The pwatch command is used to watch named productions.

.SH SEE ALSO
\fBpwatch\fR, \fBprint\fR

