##########################################################################
# File         : tsiControlPanel.tcl
# Author       : Gordon D. Baxter
# Date	       : 01/07/96
# Description  : 
#
#
##########################################################################

###
###	I.	Make the TSI Control Panel
###


proc makeTSIControlPanel { {hide 0} {agentCreationGUI tsiTextEntryACGUI} } {
  global tsi_library tsiConfig


  set tsiConfig(agentCreationGUI) $agentCreationGUI

  if $hide {
    wm withdraw .
  } else {
    wm deiconify .
  }
  . configure -relief ridge -borderwidth 5
  wm title . {Agent Control Panel}

  set x $tsiConfig(tsiControlPanelX)
  set y $tsiConfig(tsiControlPanelY)


  if {$x >= 0} { set x +$x }
  if {$y >= 0} { set y +$y }
  
  wm geometry . $x$y

  # create the menu bar
  frame .menu -relief raised -border 2
  pack .menu -side top -fill x

  menubutton .menu.file -text {File} -menu .menu.file.m

  menu .menu.file.m 
  .menu.file.m add command -label Exit -command quitSoar

  pack .menu.file -side left -padx 5

  # create a frame for the general buttons

  frame .run -borderwidth 2
  if {![info exists $tsiConfig(hasEnvironment)] || $tsiConfig(hasEnvironment) == 0} {
      button .run.step -text Step -command tsiStep-soar
      button .run.run -text Run -command tsiRun-soar
      button .run.stop -text Stop -command fake-stop-soar
      button .run.quit -text Quit -command quitSoar

      pack .run.step .run.run .run.stop .run.quit -side top -anchor n -fill both
      pack .run -side right -fill y
      
      frame .agents
      frame .agents.create -relief sunken -borderwidth 2
      label .agents.create.name -text {Name:}
      entry .agents.create.e -width 20 -relief sunken -textvariable newAgentName
      button .agents.create.b -text {Create New Agent} -relief raised \
         -command {createNewAgent $newAgentName; set newAgentName {}}

      pack .agents.create.name .agents.create.e .agents.create.b -side left
      pack .agents.create -side top -fill x

      ## no click to focus
      bind . <Enter> { focus .agents.create.e }
      bind . <Return> { .agents.create.b invoke }
   
   } else {
   
      button .run.step -text Step -command environmentStep
      button .run.run -text Run -command environmentRun 
      button .run.stop -text Stop -command environmentStop
      button .run.quit -text Quit -command quitSoar
      pack .run.step .run.run .run.stop .run.quit -side top -anchor n -fill both
      pack .run -side right -fill y

      frame .agents

      # Call the agentCreationGUI procedure...  
      #  $agentCreationGUI .agents
      tsiPullDownMenuACGUI .agents
   }

      # The rest of the window will be filled with agent frames as they
      # are registered with the control panel.
      # There will be a permanent frame at the top for commands to be
      # issued to all agents at the same time

      frame .agents.agents -relief ridge -borderwidth 5
      pack .agents.agents -side top -fill x

      label .agents.agents.label -text {Agent Menus:}
      menubutton .agents.agents.all -text {All Agents} -relief raised \
             -menu .agents.agents.all.m

      menu .agents.agents.all.m
      .agents.agents.all.m add command -label {Open All Agent Windows} \
          -command {foreach a [tsiListAgents] { \
                    set showAgent($a) 1 ;\
                    tsiSendAgent $a {wm deiconify .tsw} \
                 }}
      .agents.agents.all.m add command -label {Close All Agent Windows} \
          -command {foreach a [tsiListAgents] { \
                    set showAgent($a) 0 ;\
                    tsiSendAgent $a {wm withdraw .tsw} \
                 }}
      .agents.agents.all.m add command -label {Watch 0} \
          -command {foreach a [tsiListAgents] { \
                    tsiSendAgent $a {tsiDisplayAndSendCommand {watch 0}} \
                 }}
      .agents.agents.all.m add separator
      .agents.agents.all.m add command -label {Clear Working Memory} \
            -command {sendAllAgents init-soar}
      .agents.agents.all.m add command -label {Excise All} \
            -command {sendAllAgents {excise -all}}
      .agents.agents.all.m add command -label {Excise Chunks} \
            -command {sendAllAgents {excise -chunks}}

      pack .agents.agents.label .agents.agents.all -side left -fill y
      pack .agents -side top -padx 10 -pady 10

  # Now we need to do the specialized setup for each particular mode

    if { ![string compare $tsiConfig(mode) client] } {
      # Client mode
      
      uplevel #0 [list source [file join $tsi_library tsi-client.tcl]]
      tsiInitClientControlPanel $tsiConfig(socketioHost) \
	  $tsiConfig(socketioPort)

  } elseif { ![string compare $tsiConfig(mode) server] } {
      # Server mode
      
      
      uplevel #0 [list source [file join $tsi_library tsi-server.tcl]]
      tsiInitServerControlPanel $tsiConfig(socketioPort)

  } else {
      # Socketio Off

      uplevel #0 [list source [file join $tsi_library tsi-nosio.tcl]]
           
      
  }

  if $hide {
    createNewAgent soar
    wm withdraw .
    if {$tcl_platform(platform) == {macintosh}} {console hide}
  } else {
    wm deiconify .
  }

}


proc tsiPullDownMenuACGUI { agentFrame } {

    frame $agentFrame.create -borderwidth 2
    label $agentFrame.create.dirLabel -text {Directory: }
    
    tsiLoadAgentNames $agentFrame.create.files
    tsiLoadSubDirectories $agentFrame.create.subdirs $agentFrame.create.files
    
    label $agentFrame.create.fileLabel -text {File: }

    button $agentFrame.create.createAgentButton -text {Create New Agent} \
	-relief raised -command {createNewAgent $tsiCurrentAgentSourceFile}

    pack $agentFrame.create -side top -fill x
    pack $agentFrame.create.dirLabel $agentFrame.create.subdirs \
	$agentFrame.create.fileLabel $agentFrame.create.files \
	$agentFrame.create.createAgentButton -side left
}


proc tsiTextEntryACGUI { agentFrame } {


   frame $agentFrame.create  -borderwidth 2
   label $agentFrame.create.name -text {Name:}

   entry $agentFrame.create.agentNameEntry -width 20 \
       -relief sunken -textvariable newAgentName
   button $agentFrame.create.createAgentButton \
       -text {Create New Agent} -relief raised \
       -command {createNewAgent $newAgentName; set newAgentName {}}
   

   pack $agentFrame.create.name $agentFrame.create.agentNameEntry \
        $agentFrame.create.createAgentButton -side left
   pack $agentFrame.create -side top -fill x


   ## no click to focus
   bind . <Enter> { focus $agentFrame.create.agentNameEntry }
   bind . <Return> { $agentFrame.create.agentCreationButton invoke }

}

proc tsiPullDownMenuACGUI { agentFrame } {

    frame $agentFrame.create -borderwidth 2
    label $agentFrame.create.dirLabel -text {Directory: }
    
    tsiLoadAgentNames $agentFrame.create.files
    tsiLoadSubDirectories $agentFrame.create.subdirs $agentFrame.create.files
    
    label $agentFrame.create.fileLabel -text {File: }

    button $agentFrame.create.createAgentButton -text {Create New Agent} \
	-relief raised -command {createNewAgent $tsiCurrentAgentSourceFile}

    pack $agentFrame.create -side top -fill x
    pack $agentFrame.create.dirLabel $agentFrame.create.subdirs \
	$agentFrame.create.fileLabel $agentFrame.create.files \
	$agentFrame.create.createAgentButton -side left
}

proc tsiLoadSubDirectories { menuName fileMenuName } {
    global tsiSimulatorPath tsiCurrentAgentSourceDir

    
    if { $tsiSimulatorPath == [pwd] } {
	set dirs [list .]
	foreach file [glob -nocomplain *] {
	    if { [file isdirectory $file] } {
		lappend dirs [file join . [file tail $file]]
	    }
	}

    } else {
	set currDir [file join . [file tail [pwd]]]
	set dirs [list $currDir [file join $currDir ..]]
    }
       
    if [winfo exists $menuName] {
	[$menuName cget -menu] delete 0 end
	foreach i $dirs {
	    puts "Adding $i"
	    [$menuName cget -menu] add command -label $i \
		-command [list tsiChangeAgentSourceDir $i $menuName $fileMenuName]
	}

	set tsiCurrentAgentSourceDir [lindex $dirs 0]
	
    } else {
	eval tk_optionMenu $menuName tsiCurrentAgentSourceDir $dirs
	foreach dir $dirs {
	    [$menuName cget -menu] entryconfigure $dir -command \
		[list tsiChangeAgentSourceDir $dir $menuName $fileMenuName]
	}
    }
    
}

proc tsiChangeAgentSourceDir { newDir dirMenuName fileMenuName } {
    global tsiSimulatorPath

    set oldDir [pwd]
    cd [file join $tsiSimulatorPath $newDir]

    if { $oldDir == [pwd] } { return }

    tsiLoadAgentNames $fileMenuName
    tsiLoadSubDirectories $dirMenuName $fileMenuName


}

    

proc tsiLoadAgentNames { menuName } {
    global tsiCurrentAgentSourceFile 

    set allNames [glob -nocomplain {*.soar}]
    set names ""
    
    foreach i $allNames {
	lappend names [file rootname [file tail $i]]
    }

    if { $names == {} } {
	
	if [winfo exists $menuName] {
	    [$menuName cget -menu] delete 0 end
	    [$menuName cget -menu] add command -label {*NONE*} -command {}
	} else {
	    tk_optionMenu $menuName {*NONE*}
	    [$menuName cget -menu] entryconfigure {*NONE*} -command {}
	}
	$menuName configure -state disabled
	set currentName {}
    } else {
	if [winfo exists $menuName] {
	    [$menuName cget -menu] delete 0 end
	    foreach i $names {
		[$menuName cget -menu] add command -label $i \
		    -command [list set tsiCurrentAgentSourceFile $i]
	    }
	    if { ![info exists currentName] || \
		     ([lsearch $names $tsiCurrentAgentSourceFile] < 0)} {
		set tsiCurrentAgentSourceFile [lindex $names 0]
	    } 
	    $menuName configure -state normal
	} else {
	    eval tk_optionMenu $menuName tsiCurrentAgentSourceFile $names
	}
    }
}



###
###	II.	Helper functions
###

### For Soar 7.1, assume that all agents are in slave interpreters
### to the controller.  So if people try to get creative with their
### interpreter hierarchies, this will fail.
proc tsiListAgents {} {
   if [catch {set x [interp slaves]} m] {
      error "Cannot compute list of agents: $m"
   }
   return $x
}

### need to worry about having whitespace in the agent name...that
### can break things RMJ
proc createNewAgent {name {filepath ""} {filename ""}} {
   global auto_path tsi_library soar_library soar_doc_dir tsiConfig \
       localAgents tsiSuppressMenus
   
   if [catch "interp create $name"] {
      error "Cannot create agent named '$name'"
   }

    if [info exists localAgents] { 
	lappend localAgents $name 
    } else {
	set localAgents $name
    }

   load {} Tk $name
   $name eval [list set auto_path $auto_path]
   $name eval {package require Soar}
   $name eval [list set soar_library $soar_library]
   $name eval [list set tsi_library $tsi_library]
   $name eval [list array set tsiSuppressMenus [array get tsiSuppressMenus]]
   if [info exists soar_doc_dir] {
      $name eval [list set soar_doc_dir $soar_doc_dir]
   }
   ### Configure the agent for the TSI
   set id [array startsearch tsiConfig]
   while {[array anymore tsiConfig $id]} {
      set elm [array nextelement tsiConfig $id]
      $name eval [list set tsiConfig($elm) $tsiConfig($elm)]
   }
   $name eval [list set tcl_interactive 1]  ;# want command completion...
   array donesearch tsiConfig $id
   $name eval [list set interp_name $name]
   $name eval [list set source_path $filepath]
   $name eval [list set source_file $filename]
   $name alias registerWithController registerAgent $name
   $name alias tsiListAgents tsiListAgents
   $name eval tsiInitAgent
	incr tsiConfig(window_count)
   
   tsiSetupAgentVars $name
 

}

proc tsiSendAgent {agent command} {
   ### Need to use {} for catch, or $command gets concat'ed into a list
   if [catch {$agent eval $command} msg] {
      error "Error sending command '$command' to agent $agent!"
   }
}

### New version using slave interpreters in Soar 7.1
proc sendAllAgents {msg} {
   foreach i [tsiListAgents] {
      if [catch {$i eval $msg} m] {
         error "Error sending message $msg to agent $i: $m"
      }
   }
}



proc registerAgent {name} {
  global showAgent tsiConfig sio_agentStatus tsiAgentInfo

    ##puts "registerAgent   (begin) -- name: $name"
   

  menubutton .agents.agents.agent$name -text $name -relief raised \
              -menu .agents.agents.agent$name.m

   ##puts "registerAgent   (begin) -- name: $name"
  menu .agents.agents.agent$name.m

  # if this is a ClientTSI interface, we add an additional checkbutton
  # to each agent menu.
    ##puts "registerAgent   (begin) -- name: $name"
    if { ![string compare $tsiConfig(mode) client] } {
	.agents.agents.agent$name.m add checkbutton -label {Online} \
	   -variable sio_agentStatus($name,online) \
	   -command "agentToggleCommand $name \$sio_agentStatus($name,online) \
                       {SetClientOnline 1} {SetClientOnline 0}"
    }
    ##puts "registerAgent   (begin) -- name: $name"
	
  .agents.agents.agent$name.m add checkbutton -label {Show Agent Window} \
       -variable showAgent($name) \
       -command "agentToggleCommand $name \$showAgent($name) \
                   {wm deiconify .tsw} {wm withdraw .tsw}"
    ##puts "registerAgent   (begin) -- name: $name"
  set showAgent($name) 1
  .agents.agents.agent$name.m add command -label {Raise Agent Window} \
       -command "tsiSendAgent $name {raise .tsw}"
  .agents.agents.agent$name.m add command -label {watch 0} \
      -command "tsiSendAgent $name {tsiDisplayAndSendCommand {watch 0}}"
  .agents.agents.agent$name.m add separator
  .agents.agents.agent$name.m add command -label {Clear Working Memory} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand init-soar}"
  .agents.agents.agent$name.m add command -label {Excise All} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand {excise -all}}"
  .agents.agents.agent$name.m add command -label {Excise Chunks} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand {excise -chunks}}"
  .agents.agents.agent$name.m add separator
  .agents.agents.agent$name.m add command -label {Destroy Agent} \
      -command "destroyAgent $name"

  pack .agents.agents.agent$name -side left -fill y

   set tsiAgentInfo($name) local
   set tsiAgentInfo($name,addFN) localAgentAddWME
   set tsiAgentInfo($name,rmFN) localAgentRemoveWME
   
  
}

proc agentToggleCommand {name t on off} {
   if $t {
      tsiSendAgent $name $on
   } else {
      tsiSendAgent $name $off
   }
}

proc destroyAgent {name} {
   if {[lsearch [tsiListAgents] $name] >= 0} {
      #bind .tsw <Destroy> ""
      #destroy .tsw
      if [catch "destroy-interpreter $name"] {
         if [catch "interp delete $name"] {
            error "Cannot delete agent '$name'!"
         }
      }
   }
   if [winfo exists .agents.agents.agent$name] {
      destroy .agents.agents.agent$name
   }
}

### Need a fake-stop-soar, because stop-soar is not defined in wish
### interpreters
proc fake-stop-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't stop!  No Soar interpreters are running!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] stop-soar
}

proc tsiRun-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't run!  No Soar interpreters exist!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] run
}
proc tsiStep-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't step!  No Soar interpreters exist!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] "run 1"
}

proc temploadWindowPreferences {} {
	global soar_library windowPrefs

	if {[file exists [file join $soar_library tsiWindowPrefs.data]]} {
		set chanID [open [file join $soar_library tsiWindowPrefs.data] r]

		gets $chanID geometryRead
		wm geometry . $geometryRead
		while {![eof $chanID]} {
			gets $chanID winLine
			if {$winLine != ""} {
			   # Window preferences are stored as windowPrefs(type,name)=geometry
			   # type is either main or tsi, name is either window name or
			   # agent name(for tsi type)
				set windowPrefs([lindex $winLine 2],[lindex $winLine 0]) [lindex $winLine 1]
				if {[lindex $winLine 2] == "main"} {
					if {[winfo exists [lindex $winLine 0]]} {
						wm geometry [lindex $winLine 0] [lindex $winLine 1]
					}
				} elseif {[interp exists [lindex $winLine 0]] &&\
   				([lsearch -exact [[lindex $winLine 0] eval [list package names]] {Tk}] != -1)} {
					[lindex $winLine 0] eval wm geometry .tsw [lindex $winLine 1]
				}
			}
		}
		close $chanID
		return 1
	} else {
	   return 0
	}
}

proc tempsaveWindowPreferences {} {
	global soar_library windowPrefs

	set chanID [open [file join $soar_library tsiWindowPrefs.data] w]
	puts $chanID [wm geometry .]

	foreach w [winfo children .] {
	   if {[winfo toplevel $w] == $w} {
      	puts $chanID "$w [wm geometry $w] main"
			set windowPrefs(main,$w) [wm geometry $w]
      }
   }

   foreach i [interp slaves] {
		if {([lsearch -exact [$i eval [list package names]] {Tk}] != -1) && \
            [$i eval winfo exists .tsw]} {
			set windowPrefs(tsi,$i) [$i eval wm geometry .tsw]
			puts $chanID "$i $windowPrefs(tsi,$i) tsi"
		}
	}
	close $chanID

}

